/* GTK - The GIMP Toolkit
 * Copyright (C) 2000 Red Hat Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Vikram Subramanian <upender@vsnl.com>
 *
 */

#include <string.h>

#include <gdk/gdkkeysyms.h>

#include "gtk/gtkintl.h"
#include "gtk/gtkimcontextsimple.h"
#include "gtk/gtkimmodule.h"

#define TAMIL_IM_ID  "tamil_im"

GType type_tamil_im = 0;

static void tamil_im_class_init (GtkIMContextSimpleClass *class);
static void tamil_im_init (GtkIMContextSimple *im_context);

static void
tamil_im_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (GtkIMContextSimpleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) tamil_im_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (GtkIMContextSimple),
    0,
    (GtkObjectInitFunc) tamil_im_init,
  };

  type_tamil_im = 
    g_type_module_register_type (module,
				 GTK_TYPE_IM_CONTEXT_SIMPLE,
				 "GtkIMContextTamil",
				 &object_info, 0);
}


/* Just an initial implementation of my own since I myself haven't used 
 * any Tamil input schemes
 */
static guint16 tamil_compose_seqs[] = {

  /* Keysyms should be in ascending order??!! */
  GDK_1,        0,      0,      0,      0,      0x0BE7, 
  GDK_2,        0,      0,      0,      0,      0x0BE8, 
  GDK_3,        0,      0,      0,      0,      0x0BE9, 
  GDK_4,        0,      0,      0,      0,      0x0BEA, 
  GDK_5,        0,      0,      0,      0,      0x0BEB, 
  GDK_6,        0,      0,      0,      0,      0x0BEC, 
  GDK_7,        0,      0,      0,      0,      0x0BED, 
  GDK_8,        0,      0,      0,      0,      0x0BEE, 
  GDK_9,        0,      0,      0,      0,      0x0BEF, 
  
  
  GDK_A,	0,	0,	0,	0,	0x0B86, /* LETTER AA */
  GDK_C,	0,	0,	0,	0,	0x0BB7, /* LETTER SHA */
  GDK_E,	0,	0,	0,	0,	0x0B8F, /* LETTER EE */
  GDK_I,	0,	0,	0,	0,	0x0B88, /* LETTER II */
  GDK_L,	0,	0,	0,	0,	0x0BB3, /* LETTER LLA */
  GDK_N,	0,	0,	0,	0,	0x0BA3, /* LETTER NNA */
  GDK_O,	0,	0,	0,	0,	0x0B93, /* LETTER OO */
  GDK_R,	0,	0,	0,	0,	0x0BB1, /* LETTER RRA */
  GDK_U,	0,	0,	0,	0,	0x0B8A, /* LETTER UU */
  GDK_W,	0,	0,	0,	0,	0x0B94, /* LETTER AU */
  GDK_Y,	0,	0,	0,	0,	0x0B9E, /* LETTER NYA */
  
  GDK_backslash,0,    0,        0,      0,      0x0BA9, /* LETTER NNNA*/

  GDK_a,	0,	0,	0,	0,	0x0B85, /* LETTER A  */
  GDK_c,	0,	0,	0,	0,	0x0B9A, /* LETTER CA  */
  GDK_d,	0,	0,	0,	0,	0x0B9F, /* LETTER TTA */
  GDK_e,	0,	0,	0,	0,	0x0B8E, /* LETTER E  */
  GDK_g,	0,	0,	0,	0,	0x0B99, /* LETTER NGA */
  GDK_h,	0,	0,	0,	0,	0x0BB9, /* LETTER HA  */
  GDK_i,	0,	0,	0,	0,	0x0B87, /* LETTER I  */
  GDK_j,	0,	0,	0,	0,	0x0B9C, /* LETTER JA  */  
  GDK_k,	0,	0,	0,	0,	0x0B95, /* LETTER KA  */
  GDK_l,	0,	0,	0,	0,	0x0BB2, /* LETTER LA  */
  GDK_m,	0,	0,	0,	0,	0x0BAE, /* LETTER MA  */
  GDK_n,	0,	0,	0,	0,	0x0BA8, /* LETTER NA  */
  GDK_o,	0,	0,	0,	0,	0x0B92, /* LETTER O  */
  GDK_p,	0,	0,	0,	0,	0x0BAA, /* LETTER PA  */
  GDK_q,	0,	0,	0,	0,	0x0B83, /* VISARGA - AK */
  GDK_r,	0,	0,	0,	0,	0x0BB0, /* LETTER RA  */
  GDK_s,	0,	0,	0,	0,	0x0BB8, /* LETTER SA  */
  GDK_t,	0,	0,	0,	0,	0x0BA4, /* LETTER TA  */
  GDK_u,	0,	0,	0,	0,	0x0B89, /* LETTER U  */
  GDK_v,	0,	0,	0,	0,	0x0BB5, /* LETTER VA  */
  GDK_w,	0,	0,	0,	0,	0x0B90, /* LETTER AI */

  /* USe x for getting modifers */
  GDK_x,    GDK_A,  0,      0,      0,      0x0BBE, /* VOWEL SIGN AA */
  GDK_x,    GDK_E,  0,      0,      0,      0x0BC7, /* VOWEL SIGN EE */
  GDK_x,    GDK_I,  0,      0,      0,      0x0BC0, /* VOWEL SIGN II */
  GDK_x,    GDK_O,  0,      0,      0,      0x0BCB, /* VOWEL SIGN OO */
  GDK_x,    GDK_U,  0,      0,      0,      0x0BC2, /* VOWEL SIGN UU */
  GDK_x,    GDK_W,  0,      0,      0,      0x0BCC, /* VOWEL SIGN AU */
  
  /* GDK_x,    GDK_a,GDK_u,    0,      0,   0x0BD7,*/ /* AU Length mark */
  GDK_x,    GDK_e,  0,      0,      0,      0x0BC6, /* VOWEL SIGN E  */
  GDK_x,    GDK_i,  0,      0,      0,      0x0BBF, /* VOWEL SIGN I  */
  GDK_x,    GDK_o,  0,      0,      0,      0x0BCA, /* VOWEL SIGN O  */
  GDK_x,    GDK_q,  0,      0,      0,      0x0BCD, /* SIGN VIRAMA   */
  GDK_x,    GDK_u,  0,      0,      0,      0x0BC1, /* VOWEL SIGN U  */
  GDK_x,    GDK_w,  0,      0,      0,      0x0BC8, /* VOWEL SIGN AI */
  

  GDK_y,	0,	0,	0,	0,	0x0BAF, /* LETTER YA  */
  GDK_z,	0,	0,	0,	0,	0x0BB4  /* LETTER LLLA - ZHA */

  
};

static void
tamil_im_class_init (GtkIMContextSimpleClass *class)
{
}

static void
tamil_im_init (GtkIMContextSimple *im_context)
{
  gtk_im_context_simple_add_table (im_context,
				   tamil_compose_seqs,
				   4,
				   G_N_ELEMENTS (tamil_compose_seqs) / (4 + 2));
}

static const GtkIMContextInfo tamil_im_info = { 
  TAMIL_IM_ID,		           /* ID */
  N_("Tamil Input Method"),        /* Human readable name */
  "gtk+",			   /* Translation domain */
   GTK_LOCALEDIR,		   /* Dir for bindtextdomain (not strictly needed for "gtk+") */
  ""			           /* Languages for which this module is the default */
};

static const GtkIMContextInfo *info_list[] = {
  &tamil_im_info
};

void
im_module_init (GTypeModule *module)
{
  tamil_im_register_type (module);
}

void 
im_module_exit (void)
{
}

void 
im_module_list (const GtkIMContextInfo ***contexts,
		int                      *n_contexts)
{
  *contexts = info_list;
  *n_contexts = G_N_ELEMENTS (info_list);
}

GtkIMContext *
im_module_create (const gchar *context_id)
{
  if (strcmp (context_id, TAMIL_IM_ID) == 0)
    return GTK_IM_CONTEXT (gtk_type_new (type_tamil_im));
  else
    return NULL;
}
